<?php

namespace InSegment\WorkersFramework\Console\Commands;

use Shavshukov\EventAppBase\Console\AppCommand;

use InSegment\WorkersFramework\Interfaces\ConnectionListInterface;
use InSegment\WorkersFramework\Providers\QueueConnectionServiceProvider;
use InSegment\WorkersFramework\Services\WorkerRunner;

class Work extends AppCommand
{
    
    /**
     * {@inheritDoc}
     */
    public function usage(): string
    {
        return 'Start a worker process';
    }

    /**
     * {@inheritDoc}
     */
    public function handle(array $arguments)
    {
        $workerName = $arguments[0] ?? null;
        echo "Starting worker {$workerName}\n";
        
        if (!$workerName) {
            echo "Please, specify the worker name";
            return $this->app->stop();
        }
        
        $this->app->registerProviders([
            new QueueConnectionServiceProvider($this->app),
        ], $this->app);
        
        return $this->app->service('queue-connections-list')->then(function (ConnectionListInterface $connections) use ($workerName) {
            return (new WorkerRunner($this->app, $connections, $workerName))->runWorker();
        });
    }

    /**
     * {@inheritDoc}
     */
    public function help(string $entryFileName): string
    {
        return implode("\n", [
            "Usage: {$entryFileName} work <worker name> [<args>]\n",
            'Start a worker process with arguments'
        ]);
    }
    
}
