<?php

namespace InSegment\WorkersFramework\Services;

use React\Promise;

use InSegment\WorkersFramework\Interfaces\QueueMessageInterface;
use InSegment\WorkersFramework\Interfaces\JobListInterface;
use InSegment\WorkersFramework\Interfaces\QueueInterface;
use InSegment\WorkersFramework\Models\JobHeaders;

class WorkerConsumer extends AbstractConsumer
{
    
    /**
     * JobList instance
     *
     * @var \InSegment\WorkersFramework\Interfaces\JobListInterface
     */
    protected $jobList;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Services\QueueInterface $queue
     * @param \InSegment\WorkersFramework\Interfaces\JobListInterface $jobList
     */
    public function __construct(QueueInterface $queue, JobListInterface $jobList)
    {
        parent::__construct($queue);
        $this->jobList = $jobList;
    }
    
    /**
     * Move message from the main queue to the worker queue and acknowledge the original one
     * 
     * @param \InSegment\WorkersFramework\Interfaces\QueueMessageInterface $message
     * @return mixed
     */
    public function receiveMessage(QueueMessageInterface $message)
    {
        $headers = JobHeaders::extend($message->getHeaders());
        $headersValidation = $headers->validate();
        
        if (!$headersValidation->isValid()) {
            $printNotFoundFields = implode(', ', $headersValidation->getInvalidFields());
            echo "Job header mandatory fields not found: {$printNotFoundFields}, message will be ignored\n";
            return $message->acknowlege();
        }
        
        $jobName = $headers->getJobName();
        $jobConfig = $this->jobList->getJob($jobName);
        
        if ($jobConfig === null) {
            echo "The configuration doesn't have class mapping for job named {$jobName}, message will be ignored\n";
            return $message->acknowlege();
        }
        
        $this->trigger('job_start', $message);
        return Promise\resolve($jobConfig->createInstance()->process($message))->then(function ($result) use ($message) {
            $this->trigger('job_end', $message, $result);
            
            if ($result !== false) {
                return $message->acknowlege();
            }
            
            return $result;
        });
    }

}
